@@ -400,6 +400,20 @@ ldif_address_book_need_reload(LibBalsaAddressBookLdif *ab)
 	return FALSE;
 }
 
+static gboolean
+starts_from(const gchar *str, const gchar *filter_hi)
+{
+    if(!str) return FALSE;
+    while(*str && *filter_hi &&
+          g_unichar_toupper(g_utf8_get_char(str)) == 
+          g_utf8_get_char(filter_hi)) {
+        str       = g_utf8_next_char(str);
+        filter_hi = g_utf8_next_char(filter_hi);
+    }
+           
+    return *filter_hi == '\0';
+}
+
 static LibBalsaABErr
 libbalsa_address_book_ldif_load(LibBalsaAddressBook * ab, 
                                 const gchar *filter,
@@ -407,15 +421,24 @@ libbalsa_address_book_ldif_load(LibBalsaAddressBook * ab,
                                 gpointer closure)
 {
     GList *lst;
+    int len = filter ? strlen(filter) : 0;
+    gchar *filter_hi = NULL;
 
     if(!load_ldif_file(ab)) return LBABERR_CANNOT_READ;
+    if(len)
+        filter_hi = g_utf8_strup(filter, -1);
 
     for (lst = LIBBALSA_ADDRESS_BOOK_LDIF(ab)->address_list; 
 	 lst; lst = g_list_next(lst)) {
-	if (callback)
-	    callback(ab, LIBBALSA_ADDRESS(lst->data), closure);
+        LibBalsaAddress *adr = LIBBALSA_ADDRESS(lst->data);
+        if(callback &&(!len ||
+                       starts_from(adr->last_name, filter_hi) ||
+                       starts_from(adr->full_name, filter_hi) ) )
+	    callback(ab, adr, closure);
     }
     if(callback) callback(ab, NULL, closure);
+    if(len)
+        g_free(filter_hi);
     return LBABERR_OK;
 }
 
@@ -625,7 +648,8 @@ libbalsa_address_book_ldif_add_address(LibBalsaAddressBook * ab,
     LibBalsaABErr res = LBABERR_OK;
 
     load_ldif_file(ab); /* Ignore error if any; we may be adding */
-                        /* the first entry in the book. */    
+                        /* the first entry in the book. */
+    
     
     for(list = LIBBALSA_ADDRESS_BOOK_LDIF(ab)->address_list;
 	list;
