@@ -22,6 +22,9 @@
 #include "config.h"
 
 #include <gnome.h>
+#ifdef HAVE_GPGME
+#  include "rfc3156.h"
+#endif
 #include "identity.h"
 #include "information.h"
 
@@ -97,6 +100,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
 #ifdef HAVE_GPGME
     ident->gpg_sign = FALSE;
     ident->gpg_encrypt = FALSE;
+    ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
 #endif
 }
 
@@ -279,23 +283,6 @@ libbalsa_identity_set_sig_prepend(LibBalsaIdentity* ident, gboolean prepend)
 }
 
 
-#ifdef HAVE_GPGME
-void
-libbalsa_identity_set_gpg_sign(LibBalsaIdentity* ident, gboolean sign)
-{
-    g_return_if_fail(ident != NULL);
-    ident->gpg_sign = sign;
-}
-
-
-void
-libbalsa_identity_set_gpg_encrypt(LibBalsaIdentity* ident, gboolean encrypt)
-{
-    g_return_if_fail(ident != NULL);
-    ident->gpg_encrypt = encrypt;
-}
-#endif
-
 /* Used by both dialogs: */
 
 /* Widget padding: */
@@ -511,6 +498,18 @@ static void md_response_cb(GtkWidget * dialog, gint response,
 static void md_name_changed(GtkEntry * name, GtkTreeView * tree);
 static void md_sig_path_changed(GtkEntry * sig_path, GObject * dialog);
 
+#ifdef HAVE_GPGME
+static void add_show_menu(const char* label, gint value, GtkWidget* menu);
+static void ident_dialog_add_option_menu(GtkWidget * table, gint row,
+                                         GtkDialog * dialog,
+                                         const gchar * label_name,
+                                         const gchar * menu_key);
+static gint ident_dialog_get_menu(GtkDialog * dialog, const gchar * key);
+static void display_frame_set_menu(GtkDialog * dialog, const gchar* key,
+                                   gint * value);
+#endif /* HAVE_GPGME */
+
+
 /* Callback for the "toggled" signal of the "Default" column. */
 static void
 toggle_cb(GtkWidget * dialog, gchar * path)
@@ -584,7 +583,8 @@ libbalsa_identity_tree(GCallback toggled_cb, gpointer toggled_data,
  */
 static GtkWidget* 
 libbalsa_identity_config_frame(GList** identities,
-			       LibBalsaIdentity** defid, GtkWidget * dialog)
+			       LibBalsaIdentity** defid, GtkWidget * dialog,
+                               void (*cb)(gpointer), gpointer data)
 {
     GtkWidget* config_frame = gtk_frame_new(NULL);
     GtkWidget *tree;
@@ -597,6 +597,8 @@ libbalsa_identity_config_frame(GList** identities,
                      G_CALLBACK(set_default_ident_cb), NULL);
     g_object_set_data(G_OBJECT(tree), "identities", identities);
     g_object_set_data(G_OBJECT(tree), "default-id", defid);
+    g_object_set_data(G_OBJECT(tree), "callback", cb);
+    g_object_set_data(G_OBJECT(tree), "cb-data",  data);
 
     gtk_container_add(GTK_CONTAINER(config_frame), tree);
 
@@ -732,6 +734,8 @@ new_ident_cb(GtkTreeView * tree, GtkWidget * dialog)
     LibBalsaIdentity *ident;
     GList **identities;
     GtkWidget *name_entry;
+    void (*cb)(gpointer) = g_object_get_data(G_OBJECT(tree), "callback");
+    gpointer data        = g_object_get_data(G_OBJECT(tree), "cb-data");
 
     /* Save any changes to current identity; if it's not valid, just
      * return. */
@@ -747,6 +751,7 @@ new_ident_cb(GtkTreeView * tree, GtkWidget * dialog)
 
     name_entry = g_object_get_data(G_OBJECT(dialog), "identity-name");
     gtk_widget_grab_focus(name_entry);
+    cb(data);
 }
 
 
@@ -759,7 +764,11 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
 {
      
     GtkWidget* frame = gtk_frame_new(NULL);
+#ifdef HAVE_GPGME
+    GtkWidget *table = gtk_table_new(18, 2, FALSE);
+#else
     GtkWidget *table = gtk_table_new(15, 2, FALSE);
+#endif
     gint row = 0;
     GObject *name;
     GObject *sig_path;
@@ -811,17 +820,23 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
 
 #ifdef HAVE_GPGME
     ident_dialog_add_checkbutton(table, row++, dialog, 
-                                 _("GPG sign by default"),
+                                 _("sign messages by default"),
                                  "identity-gpgsign");
     gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
 							  "identity-gpgsign")),
 			     TRUE);
     ident_dialog_add_checkbutton(table, row++, dialog,
-                                 _("GPG encrypt by default"),
+                                 _("encrypt messages by default"),
                                  "identity-gpgencrypt");
     gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
 							  "identity-gpgencrypt")),
 			     TRUE);
+    ident_dialog_add_option_menu(table, row++, dialog,
+				 _("default crypto protocol"),
+				 "identity-crypt-protocol");
+    gtk_widget_set_sensitive(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),
+							  "identity-crypt-protocol")),
+			     TRUE);
 #endif
 
     name = g_object_get_data(G_OBJECT(dialog), "identity-name");
@@ -1015,6 +1030,7 @@ ident_dialog_update(GtkDialog* dlg)
 #ifdef HAVE_GPGME
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
+    id->crypt_protocol  = ident_dialog_get_menu(dlg, "identity-crypt-protocol");
 #endif
    
     return TRUE;
@@ -1086,6 +1102,8 @@ identity_delete_selected(GtkTreeView * tree, GtkWidget * dialog)
     GtkTreePath *path;
     LibBalsaIdentity *ident;
     GList **identities;
+    void (*cb)(gpointer) = g_object_get_data(G_OBJECT(tree), "callback");
+    gpointer data        = g_object_get_data(G_OBJECT(tree), "cb-data");
 
     /* Save the path to the current row. */
     if (!gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -1108,6 +1126,7 @@ identity_delete_selected(GtkTreeView * tree, GtkWidget * dialog)
     }
     gtk_tree_path_free(path);
     gtk_widget_grab_focus(GTK_WIDGET(tree));
+    cb(data);
 }
 
 /* 
@@ -1170,9 +1189,15 @@ help_ident_cb(void)
     }
 }
 
+/* libbalsa_identity_config_dialog displays an identity management
+   dialog. The dialog has a specified parent, existing list of
+   identites, the default one. Additionally, a callback is passed that
+   will be executed when the identity list is modified: new entries
+   are added or other entries are removed. */
 void
 libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
-				LibBalsaIdentity **default_id)
+				LibBalsaIdentity **default_id,
+                                void (*changed_cb)(gpointer))
 {
     static GtkWidget *dialog = NULL;
     GtkWidget* frame;
@@ -1189,7 +1214,7 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
 
     dialog =
         gtk_dialog_new_with_buttons(_("Manage Identities"),
-                                    parent,
+                                    parent, /* must NOT be modal */
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_STOCK_HELP, IDENTITY_RESPONSE_HELP,
                                     GTK_STOCK_NEW, IDENTITY_RESPONSE_NEW,
@@ -1197,7 +1222,8 @@ libbalsa_identity_config_dialog(GtkWindow *parent, GList **identities,
                                     GTK_STOCK_CLOSE, IDENTITY_RESPONSE_CLOSE,
                                     NULL);
 
-    frame = libbalsa_identity_config_frame(identities, default_id, dialog);
+    frame = libbalsa_identity_config_frame(identities, default_id, dialog,
+                                           changed_cb, parent);
     tree = GTK_TREE_VIEW(gtk_bin_get_child(GTK_BIN(frame)));
 
     g_signal_connect(G_OBJECT(dialog), "response",
@@ -1312,6 +1338,8 @@ display_frame_update(GtkDialog * dialog, LibBalsaIdentity* ident)
                               ident->gpg_sign);    
     display_frame_set_boolean(dialog, "identity-gpgencrypt", 
                               ident->gpg_encrypt);    
+    display_frame_set_menu(dialog, "identity-crypt-protocol",
+			   &ident->crypt_protocol);
 #endif
 }
 
@@ -1386,6 +1414,7 @@ libbalsa_identity_new_config(const gchar* prefix, const gchar* name)
 #ifdef HAVE_GPGME
     ident->gpg_sign = gnome_config_get_bool("GpgSign");
     ident->gpg_encrypt = gnome_config_get_bool("GpgEncrypt");
+    ident->crypt_protocol = gnome_config_get_int("CryptProtocol=16");
 #endif
 
     gnome_config_pop_prefix();
@@ -1421,8 +1450,120 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* prefix)
 #ifdef HAVE_GPGME
     gnome_config_set_bool("GpgSign", ident->gpg_sign);
     gnome_config_set_bool("GpgEncrypt", ident->gpg_encrypt);
+    gnome_config_set_int("CryptProtocol", ident->crypt_protocol);
 #endif
 
     gnome_config_pop_prefix();
 }
 
+
+#ifdef HAVE_GPGME
+/* collected helper stuff for GPGME support */
+
+void
+libbalsa_identity_set_gpg_sign(LibBalsaIdentity* ident, gboolean sign)
+{
+    g_return_if_fail(ident != NULL);
+    ident->gpg_sign = sign;
+}
+
+
+void
+libbalsa_identity_set_gpg_encrypt(LibBalsaIdentity* ident, gboolean encrypt)
+{
+    g_return_if_fail(ident != NULL);
+    ident->gpg_encrypt = encrypt;
+}
+
+
+void
+libbalsa_identity_set_crypt_protocol(LibBalsaIdentity* ident, gint protocol)
+{
+    g_return_if_fail(ident != NULL);
+    ident->crypt_protocol = protocol;
+}
+
+
+static void
+add_show_menu(const char* label, gint value, GtkWidget* menu)
+{
+    GtkWidget *menu_item = gtk_menu_item_new_with_label(label);
+
+    gtk_widget_show(menu_item);
+    g_object_set_data(G_OBJECT(menu_item), "identity-value",
+                      GINT_TO_POINTER(value));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+}
+
+
+/*
+ * Add an option menu to the given dialog with a label next to it
+ * explaining the contents.  A reference to the entry is stored as
+ * object data attached to the dialog with the given key.
+ */
+static void
+ident_dialog_add_option_menu(GtkWidget * table, gint row, GtkDialog * dialog,
+                             const gchar * label_name, const gchar * menu_key)
+{
+    GtkWidget *label;
+    GtkWidget *opt_menu;
+    GtkWidget *menu;
+
+    label = gtk_label_new_with_mnemonic(label_name);
+    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
+
+    opt_menu = gtk_option_menu_new ();
+    gtk_table_attach_defaults(GTK_TABLE(table), opt_menu, 1, 2, row, row + 1);
+    g_object_set_data(G_OBJECT(dialog), menu_key, opt_menu);
+
+    menu = gtk_menu_new();
+    add_show_menu(_("GnuPG using MIME mode"), LIBBALSA_PROTECT_RFC3156, menu);
+    add_show_menu(_("GnuPG using OpenPGP mode"), LIBBALSA_PROTECT_OPENPGP, menu);
+#ifdef HAVE_SMIME
+    add_show_menu(_("S/MIME mode"), LIBBALSA_PROTECT_SMIMEV3, menu);
+#endif
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu), menu);
+}
+
+
+/*
+ * Get the value of the active option menu item
+ */
+static gint
+ident_dialog_get_menu(GtkDialog * dialog, const gchar * key)
+{
+    GtkWidget * menu;
+    gint value;
+    
+    menu = gtk_option_menu_get_menu (GTK_OPTION_MENU(g_object_get_data(G_OBJECT(dialog), key)));
+    value = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(gtk_menu_get_active (GTK_MENU(menu))), "identity-value"));
+    
+    return value;
+}
+
+
+static void
+display_frame_set_menu(GtkDialog * dialog, const gchar* key, gint * value)
+{
+    GtkOptionMenu * opt_menu =
+        GTK_OPTION_MENU(g_object_get_data(G_OBJECT(dialog), key));
+ 
+    switch (*value)
+        {
+        case LIBBALSA_PROTECT_OPENPGP:
+            gtk_option_menu_set_history(opt_menu, 1);
+            break;
+#ifdef HAVE_SMIME
+        case LIBBALSA_PROTECT_SMIMEV3:
+            gtk_option_menu_set_history(opt_menu, 2);
+            break;
+#endif
+        case LIBBALSA_PROTECT_RFC3156:
+        default:
+            gtk_option_menu_set_history(opt_menu, 0);
+            *value = LIBBALSA_PROTECT_RFC3156;
+        }
+}
+
+#endif  /* HAVE_GPGME */
