@@ -48,126 +48,7 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* BOOLEAN:POINTER (libbalsa-marshal.list:2) */
-void
-libbalsa_BOOLEAN__POINTER (GClosure     *closure,
-                           GValue       *return_value,
-                           guint         n_param_values,
-                           const GValue *param_values,
-                           gpointer      invocation_hint,
-                           gpointer      marshal_data)
-{
-  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER) (gpointer     data1,
-                                                     gpointer     arg_1,
-                                                     gpointer     data2);
-  register GMarshalFunc_BOOLEAN__POINTER callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gboolean v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 2);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_BOOLEAN__POINTER) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       g_marshal_value_peek_pointer (param_values + 1),
-                       data2);
-
-  g_value_set_boolean (return_value, v_return);
-}
-
-/* BOOLEAN:POINTER,INT,POINTER (libbalsa-marshal.list:3) */
-void
-libbalsa_BOOLEAN__POINTER_INT_POINTER (GClosure     *closure,
-                                       GValue       *return_value,
-                                       guint         n_param_values,
-                                       const GValue *param_values,
-                                       gpointer      invocation_hint,
-                                       gpointer      marshal_data)
-{
-  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_INT_POINTER) (gpointer     data1,
-                                                                 gpointer     arg_1,
-                                                                 gint         arg_2,
-                                                                 gpointer     arg_3,
-                                                                 gpointer     data2);
-  register GMarshalFunc_BOOLEAN__POINTER_INT_POINTER callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gboolean v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 4);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_BOOLEAN__POINTER_INT_POINTER) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       g_marshal_value_peek_pointer (param_values + 1),
-                       g_marshal_value_peek_int (param_values + 2),
-                       g_marshal_value_peek_pointer (param_values + 3),
-                       data2);
-
-  g_value_set_boolean (return_value, v_return);
-}
-
-/* BOOLEAN:VOID (libbalsa-marshal.list:4) */
-void
-libbalsa_BOOLEAN__VOID (GClosure     *closure,
-                        GValue       *return_value,
-                        guint         n_param_values,
-                        const GValue *param_values,
-                        gpointer      invocation_hint,
-                        gpointer      marshal_data)
-{
-  typedef gboolean (*GMarshalFunc_BOOLEAN__VOID) (gpointer     data1,
-                                                  gpointer     data2);
-  register GMarshalFunc_BOOLEAN__VOID callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gboolean v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 1);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_BOOLEAN__VOID) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       data2);
-
-  g_value_set_boolean (return_value, v_return);
-}
-
-/* INT:OBJECT (libbalsa-marshal.list:5) */
+/* INT:OBJECT (./libbalsa-marshal.list:5) */
 void
 libbalsa_INT__OBJECT (GClosure     *closure,
                       GValue       *return_value,
@@ -206,7 +87,7 @@ libbalsa_INT__OBJECT (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:OBJECT,OBJECT (libbalsa-marshal.list:6) */
+/* INT:OBJECT,OBJECT (./libbalsa-marshal.list:6) */
 void
 libbalsa_INT__OBJECT_OBJECT (GClosure     *closure,
                              GValue       *return_value,
@@ -247,7 +128,7 @@ libbalsa_INT__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* INT:POINTER,POINTER,POINTER (libbalsa-marshal.list:7) */
+/* INT:POINTER,POINTER,POINTER (./libbalsa-marshal.list:8) */
 void
 libbalsa_INT__POINTER_POINTER_POINTER (GClosure     *closure,
                                        GValue       *return_value,
@@ -290,48 +171,7 @@ libbalsa_INT__POINTER_POINTER_POINTER (GClosure     *closure,
   g_value_set_int (return_value, v_return);
 }
 
-/* POINTER:INT,POINTER (libbalsa-marshal.list:8) */
-void
-libbalsa_POINTER__INT_POINTER (GClosure     *closure,
-                               GValue       *return_value,
-                               guint         n_param_values,
-                               const GValue *param_values,
-                               gpointer      invocation_hint,
-                               gpointer      marshal_data)
-{
-  typedef gpointer (*GMarshalFunc_POINTER__INT_POINTER) (gpointer     data1,
-                                                         gint         arg_1,
-                                                         gpointer     arg_2,
-                                                         gpointer     data2);
-  register GMarshalFunc_POINTER__INT_POINTER callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gpointer v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 3);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_POINTER__INT_POINTER) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       g_marshal_value_peek_int (param_values + 1),
-                       g_marshal_value_peek_pointer (param_values + 2),
-                       data2);
-
-  g_value_set_pointer (return_value, v_return);
-}
-
-/* POINTER:OBJECT (libbalsa-marshal.list:9) */
+/* POINTER:OBJECT (./libbalsa-marshal.list:11) */
 void
 libbalsa_POINTER__OBJECT (GClosure     *closure,
                           GValue       *return_value,
@@ -370,7 +210,7 @@ libbalsa_POINTER__OBJECT (GClosure     *closure,
   g_value_set_pointer (return_value, v_return);
 }
 
-/* POINTER:POINTER,POINTER (libbalsa-marshal.list:10) */
+/* POINTER:POINTER,POINTER (./libbalsa-marshal.list:12) */
 void
 libbalsa_POINTER__POINTER_POINTER (GClosure     *closure,
                                    GValue       *return_value,
@@ -411,7 +251,7 @@ libbalsa_POINTER__POINTER_POINTER (GClosure     *closure,
   g_value_set_pointer (return_value, v_return);
 }
 
-/* POINTER:VOID (libbalsa-marshal.list:11) */
+/* POINTER:VOID (./libbalsa-marshal.list:13) */
 void
 libbalsa_POINTER__VOID (GClosure     *closure,
                         GValue       *return_value,
@@ -448,7 +288,46 @@ libbalsa_POINTER__VOID (GClosure     *closure,
   g_value_set_pointer (return_value, v_return);
 }
 
-/* VOID:INT,INT,INT,STRING (libbalsa-marshal.list:12) */
+/* VOID:INT,INT,INT (./libbalsa-marshal.list:14) */
+void
+libbalsa_VOID__INT_INT_INT (GClosure     *closure,
+                            GValue       *return_value,
+                            guint         n_param_values,
+                            const GValue *param_values,
+                            gpointer      invocation_hint,
+                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_INT_INT) (gpointer     data1,
+                                                  gint         arg_1,
+                                                  gint         arg_2,
+                                                  gint         arg_3,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__INT_INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_int (param_values + 3),
+            data2);
+}
+
+/* VOID:INT,INT,INT,STRING (./libbalsa-marshal.list:15) */
 void
 libbalsa_VOID__INT_INT_INT_STRING (GClosure     *closure,
                                    GValue       *return_value,
@@ -489,7 +368,7 @@ libbalsa_VOID__INT_INT_INT_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (libbalsa-marshal.list:13) */
+/* VOID:POINTER,INT (./libbalsa-marshal.list:16) */
 void
 libbalsa_VOID__POINTER_INT (GClosure     *closure,
                             GValue       *return_value,
