@@ -56,21 +56,18 @@
 #define NUM_DEFAULT_WIDTH 30
 #define STATUS_DEFAULT_WIDTH 16
 #define ATTACHMENT_DEFAULT_WIDTH 16
-#define FROM_DEFAULT_WIDTH 160
-#define SUBJECT_DEFAULT_WIDTH 250
+#define FROM_DEFAULT_WIDTH 128
+#define SUBJECT_DEFAULT_WIDTH 180
 #define DATE_DEFAULT_WIDTH 128
 #define SIZE_DEFAULT_WIDTH 40
 
 /* default width settings for the mailbox list columns, not fully utilized yet */
 #define MBNAME_DEFAULT_WIDTH 80
 
-#define NEWMSGCOUNT_DEFAULT_WIDTH 45
-#define TOTALMSGCOUNT_DEFAULT_WIDTH 45
+#define NEWMSGCOUNT_DEFAULT_WIDTH 20
+#define TOTALMSGCOUNT_DEFAULT_WIDTH 25
 #define INFO_FIELD_LENGTH 10
 
-/* Default colour for mailboxes with unread messages */
-#define MBLIST_UNREAD_COLOR "dark green"
-
 /*
  * Default colour for quoted text
  * oh no, I used the US spelling.
@@ -92,9 +89,9 @@
 #define DEFAULT_SUBJECT_FONT "helvetica Bold 10"
 #define DEFAULT_DATE_FORMAT "%x %X"
 #define DEFAULT_PAPER_SIZE "A4"
-#define DEFAULT_PRINT_HEADER_FONT "Times Roman 11"
-#define DEFAULT_PRINT_BODY_FONT "Courier 11"
-#define DEFAULT_PRINT_FOOTER_FONT "Times Roman 7"
+#define DEFAULT_PRINT_HEADER_FONT "Bitstream Charter Bold 12"
+#define DEFAULT_PRINT_BODY_FONT "Courier Regular 11"
+#define DEFAULT_PRINT_FOOTER_FONT "Bitstream Charter Bold Italic 8"
 #define DEFAULT_SELECTED_HDRS "from to date cc subject"
 #define DEFAULT_MESSAGE_TITLE_FORMAT "Message from %F: %s"
 #define DEFAULT_ENCODING ENC8BIT
@@ -174,7 +171,8 @@ extern struct BalsaApplication {
     gchar *smtp_certificate_passphrase;
 #endif
 
-    /* IMAP folder scanning */
+    /* folder scanning */
+    guint local_scan_depth;
     guint imap_scan_depth;
 
     BalsaWindow *main_window;
@@ -188,7 +186,7 @@ extern struct BalsaApplication {
     LibBalsaMailbox *outbox;
     LibBalsaMailbox *trash;
 
-    GNode *mailbox_nodes;
+    BalsaMailboxNode *root_node;
 
     /* timer for mm_exists callback */
     gint new_messages_timer;
@@ -210,12 +208,15 @@ extern struct BalsaApplication {
     /* automatically close mailboxes after XX minutes */
     gboolean close_mailbox_auto;
     gint close_mailbox_timeout; /* seconds */
-    /* automatically commit mailboxes after XX minutes */
-    gboolean commit_mailbox_auto;
-    gint commit_mailbox_timeout; /* seconds */
+
+    /* automatically expunge mailboxes after XX hours */
+    gboolean expunge_auto;
+    gint expunge_timeout;	/* seconds */
+
     gint check_imap;
     gint check_imap_inbox;
     gint quiet_background_check;
+    gint msg_size_limit; /* for POP mailboxes; in kB */
 
     /* GUI settings */
     gint mw_width;
@@ -257,7 +258,6 @@ extern struct BalsaApplication {
     /* Colour of mailboxes with unread messages in mailbox list */
     GdkVisual *visual;
     GdkColormap *colormap;
-    GdkColor mblist_unread_color;
 
     /* Colour of quoted text. */
     gchar *quote_regex;
@@ -277,16 +277,17 @@ extern struct BalsaApplication {
     ShownHeaders shown_headers;
     gboolean show_all_headers;
     gchar *selected_headers;
-    gchar *message_title_format;
-    LibBalsaMailboxThreadingType threading_type;
     gboolean expand_tree;
     gboolean show_mblist;
     gboolean show_notebook_tabs;
     gboolean alternative_layout;
     gboolean view_message_on_open;
-    gboolean line_length;
     gboolean pgdownmod;
     gint pgdown_percent;
+#if defined(ENABLE_TOUCH_UI)
+    gboolean do_file_format_check; /* do file format check on attaching */
+    gboolean enable_view_filter;   /* enable quick view filter */
+#endif
 
     gboolean empty_trash_on_exit;
     gboolean previewpane;
@@ -384,14 +385,13 @@ extern struct BalsaApplication {
 #ifdef HAVE_GPGME
     /* gpgme stuff */
     gboolean has_openpgp;
-#endif
+    gboolean has_smime;
+#endif 
 
     GList *folder_mru;
     GList *fcc_mru;
-    gboolean delete_immediately;
     gboolean hide_deleted;
-
-    GHashTable *mailbox_views;
+    gboolean expunge_on_close;
 
     /* use as default email client for GNOME */
     int default_client;
@@ -402,7 +402,6 @@ extern struct BalsaApplication {
 
 void balsa_app_init(void);
 void balsa_app_destroy(void);
-gboolean do_load_mailboxes(void);
 void update_timer(gboolean update, guint minutes);
 
 gchar *ask_password(LibBalsaServer * server, LibBalsaMailbox * mbox);
@@ -411,15 +410,15 @@ GtkWidget *balsa_stock_button_with_label(const char *icon,
 gboolean open_mailboxes_idle_cb(gchar * names[]);
 
 /* Search functions */
-GNode *balsa_find_mailbox(GNode * root, LibBalsaMailbox * mailbox);
-GNode *balsa_find_mbnode(GNode * root, BalsaMailboxNode * mbnode);
-GNode *balsa_find_dir(GNode * root, const gchar * path);
-BalsaMailboxNode *balsa_find_mailbox_node_by_url(const gchar * url);
+BalsaMailboxNode *balsa_find_mailbox(LibBalsaMailbox * mailbox);
+BalsaMailboxNode *balsa_find_dir(LibBalsaServer *server, const gchar * path);
+BalsaMailboxNode *balsa_find_url(const gchar * url);
 LibBalsaMailbox *balsa_find_mailbox_by_url(const gchar * url);
-
-void  balsa_remove_children_mailbox_nodes(GNode* gnode);
+gboolean balsa_find_iter_by_data(GtkTreeIter * iter, gpointer data);
 BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
 
+void  balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode);
+
 GtkWidget *create_label(const gchar * label, GtkWidget * table, gint row);
 GtkWidget *create_entry(GtkDialog *mcw, GtkWidget * table, 
 			GtkSignalFunc func, gpointer data, gint row, 
@@ -427,7 +426,4 @@ GtkWidget *create_entry(GtkDialog *mcw, GtkWidget * table,
 GtkWidget *create_check(GtkDialog *mcw, const gchar * label, 
 			GtkWidget * table, gint row, gboolean initval);
 
-void balsa_mailbox_nodes_lock(gboolean exclusive);
-void balsa_mailbox_nodes_unlock(gboolean exclusive);
-
 #endif				/* __BALSA_APP_H__ */
