@@ -70,7 +70,6 @@
 #define BALSA_PIXMAP_MENU_NEXT_FLAGGED		"balsa_menu_next_flagged"
 #define BALSA_PIXMAP_MENU_NEXT_UNREAD		"balsa_menu_next_unread"
 #define BALSA_PIXMAP_MENU_MARK_ALL		"balsa_menu_mark_all"
-#define BALSA_PIXMAP_MENU_MARK_ALL_READ		"balsa_menu_mark_all_read"
 
 #define BALSA_PIXMAP_OTHER_CLOSE		"balsa_other_close"
 
@@ -88,6 +87,7 @@
 #define BALSA_PIXMAP_INFO_REPLIED               "balsa_info_replied"
 #define BALSA_PIXMAP_INFO_NEW                   "balsa_info_new"
 #define BALSA_PIXMAP_INFO_ATTACHMENT            "balsa_info_attachment"
+#define BALSA_PIXMAP_INFO_DELETED               "balsa_info_deleted"
 #ifdef HAVE_GPGME
 #  define BALSA_PIXMAP_GPG_SIGN                 "balsa_gpg_sign"
 #  define BALSA_PIXMAP_GPG_ENCRYPT              "balsa_gpg_encrypt"
@@ -99,6 +99,7 @@
 #endif
 
 void register_balsa_pixmaps(void);
+void register_balsa_pixbufs(GtkWidget * widget);
 void balsa_icon_create(const gchar ** data, GdkPixmap ** pmap,
                        GdkBitmap ** bmap);
 #endif
