@@ -184,7 +184,7 @@ balsa_filter_run_dialog_new(LibBalsaMailbox * mbox)
 
     /* We set the dialog title */
     p->mbox=mbox;
-    libbalsa_mailbox_open(p->mbox); 
+    libbalsa_mailbox_open(p->mbox, NULL); 
     dialog_title=g_strconcat(_("Balsa Filters of Mailbox: "),
                              p->mbox->name,NULL);
     gtk_window_set_title(GTK_WINDOW(p),dialog_title);
@@ -367,7 +367,8 @@ static void
 balsa_filter_run_dispose(GObject * object)
 {
     BalsaFilterRunDialog* bfrd = BALSA_FILTER_RUN_DIALOG(object);
-    if(bfrd->mbox) libbalsa_mailbox_close(bfrd->mbox); 
+    if (bfrd->mbox)
+        libbalsa_mailbox_close(bfrd->mbox, balsa_app.expunge_on_close);
     bfrd->mbox = NULL;
     G_OBJECT_CLASS(parent_class)->dispose(object);
 }
