@@ -19,6 +19,7 @@
  * 02111-1307, USA.
  */
 
+#define _XOPEN_SOURCE 500
 #include "config.h"
 
 #include <gnome.h>
@@ -48,7 +49,7 @@
 #include "save-restore.h"
 #include "sendmsg-window.h"
 #include "information.h"
-#include "pop3.h"
+#include "imap-server.h"
 
 #include "libinit_balsa/init_balsa.h"
 
@@ -83,8 +84,8 @@ static void threads_destroy(void);
 #endif				/* BALSA_USE_THREADS */
 
 static void balsa_init(int argc, char **argv);
-static void config_init(void);
-static void mailboxes_init(void);
+static void config_init(gboolean check_only);
+static void mailboxes_init(gboolean check_only);
 static void balsa_cleanup(void);
 static gint balsa_kill_session(GnomeClient * client, gpointer client_data);
 static gint balsa_save_session(GnomeClient * client, gint phase,
@@ -101,7 +102,7 @@ gboolean initial_open_inbox(void);
 */
 static gchar *cmd_line_open_mailboxes;
 static gboolean cmd_check_mail_on_startup,
-    cmd_open_unread_mailbox, cmd_open_inbox;
+    cmd_open_unread_mailbox, cmd_open_inbox, cmd_get_stats;
 
 /* opt_attach_list: list of attachments */
 static GSList* opt_attach_list = NULL;
@@ -139,6 +140,18 @@ balsa_handle_automation_options() {
        if (cmd_open_inbox)
 	   GNOME_Balsa_Application_openInbox (app, &ev);
 
+       if (cmd_get_stats) {
+           CORBA_long unread = 0, unsent = 0;
+	   GNOME_Balsa_Application_getStats (app, &unread, &unsent, &ev);
+           printf("Unread: %ld Unsent: %ld\n", (long)unread, (long)unsent);
+       }
+
+       if (cmd_get_stats) {
+           CORBA_long unread = 0, unsent = 0;
+	   GNOME_Balsa_Application_getStats (app, &unread, &unsent, &ev);
+           printf("Unread: %ld Unsent: %ld\n", (long)unread, (long)unsent);
+       }
+
        if (cmd_line_open_mailboxes)
 	   GNOME_Balsa_Application_openMailbox (app,
 					       cmd_line_open_mailboxes,
@@ -199,7 +212,7 @@ balsa_init(int argc, char **argv)
 
 	{"checkmail", 'c', POPT_ARG_NONE,
 	 &(cmd_check_mail_on_startup), 0,
-	 N_("Get new mail on staartup"), NULL},
+	 N_("Get new mail on startup"), NULL},
 	{"compose", 'm', POPT_ARG_STRING, &(opt_compose_email),
 	 0, N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
 	{"attach", 'a', POPT_ARG_STRING, &(attachment),
@@ -212,14 +225,18 @@ balsa_init(int argc, char **argv)
 	{"open-inbox", 'i', POPT_ARG_NONE,
 	 &(cmd_open_inbox), 0,
 	 N_("Opens default Inbox on startup"), NULL},
+	{"get-stats", 's', POPT_ARG_NONE,
+	 &(cmd_get_stats), 0,
+	 N_("Prints number unread and unsent messages"), NULL},
 	{"debug-pop", 'd', POPT_ARG_NONE, &PopDebug, 0, 
 	 N_("Debug POP3 connection"), NULL},
+	{"debug-imap", 'D', POPT_ARG_NONE, &ImapDebug, 0, 
+	 N_("Debug IMAP connection"), NULL},
 	{NULL, '\0', 0, NULL, 0}	/* end the list */
     };
 
 
     context = poptGetContext(PACKAGE, argc, (const char **)argv, options, 0);
-
     while((opt = poptGetNextOpt(context)) > 0) {
         switch (opt) {
 	    case 'a':
@@ -228,6 +245,8 @@ balsa_init(int argc, char **argv)
 		break;
 	}
     }
+    poptFreeContext(context);
+
     /* Process remaining options,  */
     
     gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
@@ -282,18 +301,19 @@ check_special_mailboxes(void)
 }
 
 static void
-config_init(void)
+config_init(gboolean check_only)
 {
-    while(!config_load() || check_special_mailboxes()) {
+    while(!config_load() && !check_only) {
 	balsa_init_begin();
         config_defclient_save();
     }
 }
 
 static void
-mailboxes_init(void)
+mailboxes_init(gboolean check_only)
 {
-    if (!do_load_mailboxes()) {
+    check_special_mailboxes();
+    if (!balsa_app.inbox && !check_only) {
 	g_warning("*** error loading mailboxes\n");
 	balsa_init_begin();
         config_defclient_save();
@@ -306,9 +326,16 @@ mailboxes_init(void)
 static void
 threads_init(void)
 {
+    int status;
+    pthread_mutexattr_t attr;
     g_thread_init(NULL);
     gdk_threads_init();
-    pthread_mutex_init(&mailbox_lock, NULL);
+    
+    if( (status=pthread_mutexattr_init(&attr)) )
+        g_warning("pthread_mutexattr_init failed with %d\n", status);
+    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mailbox_lock, &attr);
+    pthread_mutexattr_destroy(&attr);
     pthread_mutex_init(&send_messages_lock, NULL);
     checking_mail = 0;
     updating_progressbar = 0;
@@ -380,28 +407,50 @@ initial_open_inbox()
     return FALSE;
 }
 
-static gint
-append_subtree_f(GNode* gn, gpointer data)
+void balsa_get_stats(long *unread, long *unsent);
+void
+balsa_get_stats(long *unread, long *unsent)
 {
-    g_return_val_if_fail(gn->data, FALSE);
-    balsa_mailbox_node_append_subtree(BALSA_MAILBOX_NODE(gn->data),
-				      gn);
-    return FALSE;
+    
+    if(balsa_app.inbox && libbalsa_mailbox_open(balsa_app.inbox, NULL) ) {
+        /* set threading type to load messages */
+        gdk_threads_enter();
+        libbalsa_mailbox_set_threading(balsa_app.inbox,
+                                       balsa_app.inbox->view->threading_type);
+        gdk_threads_leave();
+        *unread = balsa_app.inbox->unread_messages;
+        libbalsa_mailbox_close(balsa_app.inbox, FALSE);
+    } else *unread = -1;
+    if(balsa_app.draftbox && libbalsa_mailbox_open(balsa_app.outbox, NULL)){
+        *unsent = libbalsa_mailbox_total_messages(balsa_app.outbox);
+        libbalsa_mailbox_close(balsa_app.outbox, FALSE);
+    } else *unsent = -1;
 }
 
 /* scan_mailboxes:
    this is an idle handler. Expands subtrees.
-   FIXME: make sure that append_subtree_f does not use gtk.
 */
 static gboolean
 scan_mailboxes_idle_cb()
 {
+    gboolean valid;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+
     gdk_threads_enter();
-    balsa_mailbox_nodes_lock(TRUE);
-    g_node_traverse(balsa_app.mailbox_nodes, G_POST_ORDER, G_TRAVERSE_ALL, -1,
-                    append_subtree_f, NULL);
-    balsa_mailbox_nodes_unlock(TRUE);
-    balsa_mblist_repopulate(balsa_app.mblist_tree_store);
+    model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
+    /* The model contains only nodes from config. */
+    for (valid = gtk_tree_model_get_iter_first(model, &iter); valid;
+	 valid = gtk_tree_model_iter_next(model, &iter)) {
+	BalsaMailboxNode *mbnode;
+
+	gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
+	balsa_mailbox_node_append_subtree(mbnode);
+	g_object_unref(mbnode);
+    }
+    /* The root-node (typically ~/mail) isn't in the model, so its
+     * children will be appended to the top level. */
+    balsa_mailbox_node_append_subtree(balsa_app.root_node);
     gdk_threads_leave();
 
     if (cmd_open_unread_mailbox || balsa_app.open_unread_mailbox)
@@ -418,8 +467,59 @@ scan_mailboxes_idle_cb()
     if (cmd_open_inbox || balsa_app.open_inbox_upon_startup)
 	g_idle_add((GSourceFunc) initial_open_inbox, NULL);
 
+    if(cmd_get_stats) {
+        long unread, unsent;
+        balsa_get_stats(&unread, &unsent);
+        printf("Unread: %ld Unsent: %ld\n", unread, unsent);
+    }
+
     return FALSE; 
 }
+
+/* periodic_expunge_func makes sure that even the open mailboxes get
+ * expunged now and than, even if they are opened longer than
+ * balsa_app.expunge_timeout. If we did not do it, the mailboxes would in
+ * principle grow indefinetely. */
+static gboolean
+mbnode_expunge_func(GtkTreeModel *model, GtkTreePath *path,
+                    GtkTreeIter *iter)
+{
+    BalsaMailboxNode *mbnode;
+
+    g_return_val_if_fail(mbnode, FALSE);
+
+    gtk_tree_model_get(model, iter, 0, &mbnode, -1);
+    if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)) {
+        time_t tm = time(NULL);
+        if(tm-mbnode->last_use > balsa_app.expunge_timeout)
+            libbalsa_mailbox_sync_storage(mbnode->mailbox, TRUE);
+    }
+    g_object_unref(mbnode);
+
+    return FALSE;
+}
+
+static gboolean
+periodic_expunge_cb(void)
+{
+#if !defined(ENABLE_TOUCH_UI)
+    /* should we enforce expunging now and then? Perhaps not... */
+    if(!balsa_app.expunge_auto) return TRUE;
+#endif
+
+    libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
+                         _("Compressing mail folders..."));
+    /* make sure it is shown before we continue */
+    gdk_threads_enter();
+    gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
+			   (GtkTreeModelForeachFunc)mbnode_expunge_func,
+			   NULL);
+    gdk_threads_leave();
+    /* purge imap cache? leave 15MB */
+    libbalsa_imap_purge_temp_dir(15*1024*1024);
+    return TRUE; /* do it later as well */
+}
+
 /* -------------------------- main --------------------------------- */
 int
 main(int argc, char *argv[])
@@ -450,7 +550,7 @@ main(int argc, char *argv[])
     gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
     gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
 #endif /* ENABLE_NLS */
-#endif /* HAVE_GPGME */
+#endif
 
 #ifdef BALSA_USE_THREADS
     /* initiate thread mutexs, variables */
@@ -475,22 +575,41 @@ main(int argc, char *argv[])
     libbalsa_filters_set_filter_list(&balsa_app.filters);
     
     /* checking for valid config files */
-    config_init();
-    /* load mailboxes */
-    mailboxes_init();
+    config_init(cmd_get_stats);
+
+    libbalsa_mailbox_view_table =
+	g_hash_table_new_full(g_str_hash, g_str_equal,
+			      (GDestroyNotify) g_free,
+			      (GDestroyNotify) libbalsa_mailbox_view_free);
+    config_views_load();
 
     default_icon = balsa_pixmap_finder("balsa_icon.png");
-    gnome_window_icon_set_default_from_file(default_icon);
-    g_free(default_icon);
+    if(default_icon) { /* may be NULL for developer installations */
+        gnome_window_icon_set_default_from_file(default_icon);
+        g_free(default_icon);
+    }
 
     signal( SIGPIPE, SIG_IGN );
     gnome_triggers_do("", "program", "balsa", "startup", NULL);
 
     window = balsa_window_new();
     balsa_app.main_window = BALSA_WINDOW(window);
+    g_object_add_weak_pointer(G_OBJECT(window),
+			      (gpointer) &balsa_app.main_window);
     g_signal_connect(G_OBJECT(window), "destroy",
                      G_CALLBACK(balsa_cleanup), NULL);
 
+    /* load mailboxes */
+    config_load_sections();
+    mailboxes_init(cmd_get_stats);
+
+    if(cmd_get_stats) {
+        long unread, unsent;
+        balsa_get_stats(&unread, &unsent);
+        printf("Unread: %ld Unsent: %ld\n", unread, unsent);
+        exit(0);
+    }
+
     /* session management */
     client = gnome_master_client();
     g_signal_connect(G_OBJECT(client), "save_yourself",
@@ -500,28 +619,31 @@ main(int argc, char *argv[])
 
 #ifdef HAVE_GPGME
     balsa_app.has_openpgp = 
-	libbalsa_check_crypto_engine(GPGME_PROTOCOL_OpenPGP);
-#endif
-
+        libbalsa_check_crypto_engine(GPGME_PROTOCOL_OpenPGP);
+#ifdef HAVE_SMIME
+    balsa_app.has_smime =
+       libbalsa_check_crypto_engine(GPGME_PROTOCOL_CMS);
+#else
+    balsa_app.has_smime = FALSE;
+#endif /* HAVE_SMIME */
+#endif /* HAVE_GPGME */
+    
     if (opt_compose_email || opt_attach_list) {
-	BalsaSendmsg *snd;
-	GSList *lst;
+        BalsaSendmsg *snd;
+        GSList *lst;
         gdk_threads_enter();
-	snd = sendmsg_window_new(window, NULL, SEND_NORMAL);
+        snd = sendmsg_window_new(window, NULL, SEND_NORMAL);
         gdk_threads_leave();
-	if(opt_compose_email) {
-	    if(g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
-	        sendmsg_window_process_url(opt_compose_email+7, 
-		    	sendmsg_window_set_field, snd);
-	    else sendmsg_window_set_field(snd,"to", opt_compose_email);
-	}
-	for(lst = opt_attach_list; lst; lst = g_slist_next(lst))
-	    add_attachment(GNOME_ICON_LIST(snd->attachments[1]), 
-			   lst->data, FALSE, NULL);
+        if(opt_compose_email) {
+            if(g_ascii_strncasecmp(opt_compose_email, "mailto:", 7) == 0)
+                sendmsg_window_process_url(opt_compose_email+7,
+                        sendmsg_window_set_field, snd);
+            else sendmsg_window_set_field(snd,"to", opt_compose_email);
+        }
+        for(lst = opt_attach_list; lst; lst = g_slist_next(lst))
+            add_attachment(snd, lst->data, FALSE, NULL);
 	snd->quit_on_close = FALSE;
- 
     };
-
     gtk_widget_show(window);
 
     if (cmd_check_mail_on_startup || balsa_app.check_mail_upon_startup)
@@ -529,6 +651,7 @@ main(int argc, char *argv[])
 
 
     g_idle_add((GSourceFunc) scan_mailboxes_idle_cb, NULL);
+    g_timeout_add(1801*1000, (GSourceFunc) periodic_expunge_cb, NULL);
 
     gdk_threads_enter();
     gtk_main();
@@ -537,7 +660,7 @@ main(int argc, char *argv[])
 #ifdef BALSA_USE_THREADS
     threads_destroy();
 #endif
-
+    libbalsa_imap_server_close_all_connections();
     return 0;
 }
 
@@ -574,13 +697,14 @@ balsa_cleanup(void)
            b. send cancel signal to it. 
         */
         pthread_cancel(get_mail_thread);
-        libbalsa_unlock_mutt();
         printf("Mail check thread cancelled. I know it is rough.\n");
         sleep(1);
     }
     pthread_mutex_unlock(&mailbox_lock);
 #endif
     balsa_app_destroy();
+    g_hash_table_destroy(libbalsa_mailbox_view_table);
+    libbalsa_mailbox_view_table = NULL;
 
     gnome_sound_shutdown();
 }
