@@ -39,7 +39,6 @@ typedef struct _MessageWindow MessageWindow;
 
 /* callbacks */
 static void destroy_message_window(GtkWidget * widget, MessageWindow * mw);
-static void mw_message_weak_notify(MessageWindow * mw, GObject * obj);
 static void close_message_window(GtkWidget * widget, gpointer data);
 
 static void replyto_message_cb(GtkWidget * widget, gpointer data);
@@ -91,6 +90,7 @@ static GnomeUIInfo shown_hdrs_menu[] = {
 static GnomeUIInfo file_menu[] = {
     GNOMEUIINFO_MENU_PRINT_ITEM(print_cb, NULL),
     GNOMEUIINFO_SEPARATOR,
+#define MENU_FILE_CLOSE_POS 2
     GNOMEUIINFO_MENU_CLOSE_ITEM(close_message_window, NULL),
     GNOMEUIINFO_END
 };
@@ -263,16 +263,22 @@ message_window_idle_handler(MessageWindow* mw)
     BalsaMessage *msg = BALSA_MESSAGE(mw->bmessage);
     LibBalsaMessage *message = mw->message;
     gchar *title;
+    gchar *from;
 
     gdk_threads_enter();
 
     mw->idle_handler_id = 0;
 
-    title = libbalsa_message_title(message,
-				   balsa_app.message_title_format);
+    /* set window title */
+    from = libbalsa_address_to_gchar(message->headers->from, 0);
+    title = g_strdup_printf(_("Message from %s: %s"), from,
+                            LIBBALSA_MESSAGE_GET_SUBJECT(message));
+    g_free(from);
     gtk_window_set_title(GTK_WINDOW(mw->window), title);
     g_free(title);
+
     balsa_message_set(msg, message);
+    balsa_message_set_close(msg, TRUE);
 
     if(msg && msg->treeview) {
 	if (msg->info_count > 1) {
@@ -285,8 +291,6 @@ message_window_idle_handler(MessageWindow* mw)
 	}
     }
 
-    g_object_unref(G_OBJECT(message)); 
-
     /* Update the style and message counts in the mailbox list */
     balsa_mblist_update_mailbox(balsa_app.mblist_tree_store,
                                 message->mailbox);
@@ -370,14 +374,20 @@ mw_set_message(MessageWindow *mw, LibBalsaMessage * message)
     mw->message = message;
     mw_set_buttons_sensitive(mw);
     g_object_set_data(G_OBJECT(message), BALSA_MESSAGE_WINDOW_KEY, mw);
-    g_object_weak_ref(G_OBJECT(message),
-		      (GWeakNotify) mw_message_weak_notify, mw);
 
     g_object_ref(message); /* protect from destroying */
     mw->idle_handler_id =
         g_idle_add((GSourceFunc) message_window_idle_handler, mw);
 }
 
+static void
+bindex_closed_cb(gpointer data, GObject *bindex)
+{
+    MessageWindow *mw = data;
+    mw->bindex = NULL;
+    gtk_widget_destroy(mw->window);
+}
+
 void
 message_window_new(LibBalsaMessage * message)
 {
@@ -429,7 +439,7 @@ message_window_new(LibBalsaMessage * message)
                      G_CALLBACK(size_alloc_cb), NULL);
     
     mw->bindex = balsa_find_index_by_mailbox(message->mailbox);
-    g_object_add_weak_pointer(G_OBJECT(mw->bindex), (gpointer) &mw->bindex);
+    g_object_weak_ref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
     g_signal_connect_swapped(G_OBJECT(mw->bindex), "index-changed",
 			     G_CALLBACK(mw_set_buttons_sensitive), mw);
 
@@ -483,6 +493,11 @@ message_window_new(LibBalsaMessage * message)
 
     gtk_widget_show_all(mw->window);
     mw_set_message(mw, message);
+    gtk_widget_add_accelerator(file_menu[MENU_FILE_CLOSE_POS].widget,
+			       "activate",
+			       GNOME_APP(mw->window)->accel_group,
+			       GDK_Escape, (GdkModifierType)0, 
+                               (GtkAccelFlags) 0);
 }
 
 static void
@@ -490,55 +505,32 @@ mw_clear_message(MessageWindow * mw)
 {
     if (mw->idle_handler_id) {
 	g_source_remove(mw->idle_handler_id);
-	if (mw->message)
-	    g_object_unref(mw->message);
 	mw->idle_handler_id = 0;
-    } else if (mw->message) {
+    } 
+    if (mw->message) {
 	g_object_set_data(G_OBJECT(mw->message), BALSA_MESSAGE_WINDOW_KEY,
 			  NULL);
-	g_object_weak_unref(G_OBJECT(mw->message),
-			    (GWeakNotify) mw_message_weak_notify, mw);
+	g_object_unref(mw->message);
+	mw->message = NULL;
     }
-    mw->message = NULL;
 }
 
 /* Handler for the "destroy" signal for mw->window. */
 static void
 destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 {
-    if (mw->bindex)
-	g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
-					     G_SIGNAL_MATCH_DATA, 0, 0,
-					     NULL, NULL, mw);
+    if (mw->bindex) { /* BalsaIndex still exists */
+        g_object_weak_unref(G_OBJECT(mw->bindex), bindex_closed_cb, mw);
+        g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
+                                             G_SIGNAL_MATCH_DATA, 0, 0,
+                                             NULL, NULL, mw);
+    }
 
     mw_clear_message(mw);
 
     g_free(mw);
 }
 
-/* GWeakNotify for mw->message. */
-static gboolean
-mw_destroy_window(MessageWindow * mw)
-{
-    gdk_threads_enter();
-    g_object_unref(mw->window);
-    gtk_widget_destroy(mw->window);
-    gdk_threads_leave();
-
-    return FALSE;
-}
-
-static void
-mw_message_weak_notify(MessageWindow * mw, GObject *obj)
-{
-    mw->message = NULL;
-    g_object_ref(mw->window);
-    /* Destroy the window in an idle handler, because the BalsaMessage
-     * must get its own weak notification about the message before we
-     * destroy the window. */
-    g_idle_add((GSourceFunc) mw_destroy_window, mw);
-}
-
 static void
 replyto_message_cb(GtkWidget * widget, gpointer data)
 {
@@ -662,7 +654,7 @@ show_selected_cb(GtkWidget * widget, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) data;
 
-	mw->headers_shown=HEADERS_SELECTED;
+    mw->headers_shown=HEADERS_SELECTED;
 
     reset_show_all_headers(mw);
     balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
@@ -711,7 +703,8 @@ mw_set_buttons_sensitive(MessageWindow * mw)
 				       other_unread > 0);
     balsa_toolbar_set_button_sensitive(toolbar, BALSA_PIXMAP_NEXT_FLAGGED,
 				       mailbox
-				       && mailbox->total_messages > 0);
+				       && libbalsa_mailbox_total_messages
+				       (mailbox) > 0);
 }
 
 static void
@@ -739,11 +732,17 @@ mw_set_selected(MessageWindow * mw)
 {
     GList *list;
 
+    /* Temporarily tell the BalsaMessage not to close when its message
+     * is finalized, so we can safely unref it in mw_clear_message.
+     * We'll restore the usual close-with-message behavior after setting
+     * the new message. */
+    balsa_message_set_close(BALSA_MESSAGE(mw->bmessage), FALSE);
     mw_clear_message(mw);
 
     list = balsa_index_selected_list(mw->bindex);
     if (g_list_length(list) == 1)
 	mw_set_message(mw, LIBBALSA_MESSAGE(list->data));
+    g_list_foreach(list, (GFunc) g_object_unref, NULL);
     g_list_free(list);
 }
 
