@@ -20,6 +20,8 @@
  */
 #include "config.h"
 
+#define _XOPEN_SOURCE 500
+
 #include <string.h>
 #include <gconf/gconf-client.h>
 #include <gnome.h>
@@ -43,6 +45,7 @@
 #define ADDRESS_BOOK_SECTION_PREFIX "address-book-"
 #define IDENTITY_SECTION_PREFIX "identity-"
 #define VIEW_SECTION_PREFIX "view-"
+#define VIEW_BY_URL_SECTION_PREFIX "viewByUrl-"
 
 static gint config_section_init(const char* section_prefix, 
 				gint (*cb)(const char*));
@@ -63,7 +66,6 @@ static void config_identities_load(void);
 static void check_for_old_sigs(GList * id_list_tmp);
 
 static void config_filters_load(void);
-void config_filters_save(void);
 
 #define folder_section_path(mn) \
     BALSA_MAILBOX_NODE(mn)->config_prefix ? \
@@ -82,11 +84,14 @@ void config_filters_save(void);
 
 gint config_load(void)
 {
-    if(!config_global_load())  /* initializes balsa_app.mailbox_nodes */
-	return FALSE;          /* needed in the next step             */
+    return config_global_load();
+}
+
+void
+config_load_sections(void)
+{
     config_section_init(MAILBOX_SECTION_PREFIX, config_mailbox_init);
     config_section_init(FOLDER_SECTION_PREFIX,  config_folder_init);
-    return TRUE;
 }
 
 static gint
@@ -211,13 +216,27 @@ load_toolbars(void)
 
    WARNING: may destroy mailbox.
 */
+#if defined(ENABLE_TOUCH_UI)
+#define INBOX_NAME   "In"
+#define SENTBOX_NAME "Sent"
+#define DRAFTS_NAME  "Drafts"
+#define OUTBOX_NAME  "Out"
+#else
+#define INBOX_NAME   "Inbox"
+#define SENTBOX_NAME "Sentbox"
+#define DRAFTS_NAME  "Draftbox"
+#define OUTBOX_NAME  "Outbox"
+#endif /* ENABLE_TOUCH_UI */
+#define TRASH_NAME "Trash"
+
 static gchar *specialNames[] = {
-    "Inbox", "Sentbox", "Trash", "Draftbox", "Outbox"
+    INBOX_NAME, SENTBOX_NAME, TRASH_NAME, DRAFTS_NAME, OUTBOX_NAME
 };
 void
 config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
 {
-    LibBalsaMailbox **special, *do_rescan_for = NULL;
+    LibBalsaMailbox **special;
+    BalsaMailboxNode *mbnode;
 
     g_return_if_fail(mailbox != NULL);
 
@@ -246,14 +265,22 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
                       libbalsa_mailbox_local_get_path(*special),
                       strlen(balsa_app.local_mail_directory)) != 0) 
             config_mailbox_add(*special, NULL);
-        else do_rescan_for = *special;
-	g_object_unref(G_OBJECT(*special));
+	g_object_remove_weak_pointer(G_OBJECT(*special), (gpointer) special);
+
+	mbnode = balsa_find_mailbox(*special);
+	*special = NULL;
+	balsa_mblist_mailbox_node_redraw(mbnode);
+	g_object_unref(mbnode);
     }
     config_mailbox_delete(mailbox);
     config_mailbox_add(mailbox, specialNames[which]);
 
     *special = mailbox;
-    g_object_ref(G_OBJECT(mailbox));
+    g_object_add_weak_pointer(G_OBJECT(*special), (gpointer) special);
+
+    mbnode = balsa_find_mailbox(mailbox);
+    balsa_mblist_mailbox_node_redraw(mbnode);
+    g_object_unref(mbnode);
 
     switch(which) {
     case SPECIAL_SENT: 
@@ -262,8 +289,6 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
         libbalsa_filters_set_trash(balsa_app.trash); break;
     default: break;
     }
-
-    if(do_rescan_for) balsa_mailbox_local_rescan_parent(do_rescan_for);
 }
 
 void
@@ -391,6 +416,8 @@ gint config_folder_update(BalsaMailboxNode * mbnode)
 
     key = folder_section_path(mbnode);
     res = gnome_config_has_section(key);
+    gnome_config_clean_section(key);
+    gnome_config_private_clean_section(key);
     gnome_config_push_prefix(key);
     balsa_mailbox_node_save_config(mbnode, key);
     g_free(key);
@@ -472,17 +499,20 @@ config_mailbox_init(const gchar * prefix)
     LibBalsaMailbox *mailbox;
     const gchar *key =
 	prefix + strlen(BALSA_CONFIG_PREFIX MAILBOX_SECTION_PREFIX);
-    GNode *node;
 
     g_return_val_if_fail(prefix != NULL, FALSE);
 
     mailbox = libbalsa_mailbox_new_from_config(prefix);
     if (mailbox == NULL)
 	return FALSE;
-    if (LIBBALSA_IS_MAILBOX_REMOTE(mailbox))
-        g_signal_connect(G_OBJECT(LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox)),
-                         "get-password", G_CALLBACK(ask_password),
-                         mailbox);
+    if (LIBBALSA_IS_MAILBOX_REMOTE(mailbox)) {
+	LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox);
+        libbalsa_server_connect_signals(server,
+                                        G_CALLBACK(ask_password), mailbox);
+	g_signal_connect_swapped(server, "set-host",
+                                 G_CALLBACK(config_mailbox_update),
+				 mailbox);
+    }
 
     if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
         g_signal_connect(G_OBJECT(mailbox),
@@ -495,28 +525,32 @@ config_mailbox_init(const gchar * prefix)
 	    g_list_append(balsa_app.inbox_input, 
 			  balsa_mailbox_node_new_from_mailbox(mailbox));
     } else {
-        gboolean special = TRUE;
+        LibBalsaMailbox **special = NULL;
+	BalsaMailboxNode *mbnode;
 
-	node = g_node_new(balsa_mailbox_node_new_from_mailbox(mailbox));
-        balsa_mailbox_nodes_lock(TRUE);
-	g_node_append(balsa_app.mailbox_nodes, node);
-        balsa_mailbox_nodes_unlock(TRUE);
-	if (strcmp("Inbox/", key) == 0)
-	    balsa_app.inbox = mailbox;
-	else if (strcmp("Outbox/", key) == 0)
-	    balsa_app.outbox = mailbox;
-	else if (strcmp("Sentbox/", key) == 0)
-	    balsa_app.sentbox = mailbox;
-	else if (strcmp("Draftbox/", key) == 0)
-	    balsa_app.draftbox = mailbox;
-	else if (strcmp("Trash/", key) == 0) {
-	    balsa_app.trash = mailbox;
+	mbnode = balsa_mailbox_node_new_from_mailbox(mailbox);
+	if (strcmp(INBOX_NAME "/", key) == 0)
+	    special = &balsa_app.inbox;
+	else if (strcmp(OUTBOX_NAME "/", key) == 0)
+	    special = &balsa_app.outbox;
+	else if (strcmp(SENTBOX_NAME "/", key) == 0)
+	    special = &balsa_app.sentbox;
+	else if (strcmp(DRAFTS_NAME "/", key) == 0)
+	    special = &balsa_app.draftbox;
+	else if (strcmp(TRASH_NAME "/", key) == 0) {
+	    special = &balsa_app.trash;
             libbalsa_filters_set_trash(balsa_app.trash);
-        } else
-            special = FALSE;
+	}
 
-	if (special)
-            g_object_ref(G_OBJECT(mailbox));
+	if (special) {
+	    /* Designate as special before appending to the mblist, so
+	     * that view->show gets set correctly for sentbox, draftbox,
+	     * and outbox. */
+	    *special = mailbox;
+	    g_object_add_weak_pointer(G_OBJECT(mailbox), (gpointer) special);
+	}
+
+        balsa_mblist_mailbox_node_append(NULL, mbnode);
     }
     return TRUE;
 }				/* config_mailbox_init */
@@ -531,10 +565,12 @@ config_folder_init(const gchar * prefix)
 
     g_return_val_if_fail(prefix != NULL, FALSE);
 
-    balsa_mailbox_nodes_lock(TRUE);
-    if( (folder = balsa_mailbox_node_new_from_config(prefix)) )
-	g_node_append(balsa_app.mailbox_nodes, g_node_new(folder));
-    balsa_mailbox_nodes_unlock(TRUE);
+    if( (folder = balsa_mailbox_node_new_from_config(prefix)) ) {
+	g_signal_connect_swapped(folder->server, "set-host",
+                                 G_CALLBACK(config_folder_update),
+				 folder);
+	balsa_mblist_mailbox_node_append(NULL, folder);
+    }
 
     return folder != NULL;
 }				/* config_folder_init */
@@ -553,7 +589,6 @@ static gint
 config_global_load(void)
 {
     gboolean def_used;
-    BalsaMailboxNode *root_node;
 
     config_address_books_load();
     config_identities_load();
@@ -575,7 +610,7 @@ config_global_load(void)
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "InformationMessages/");
 
     balsa_app.information_message =
-	d_get_gint("ShowInformationMessages", BALSA_INFORMATION_SHOW_NONE);
+	d_get_gint("ShowInformationMessages", BALSA_INFORMATION_SHOW_BAR);
     balsa_app.warning_message =
 	d_get_gint("ShowWarningMessages", BALSA_INFORMATION_SHOW_LIST);
     balsa_app.error_message =
@@ -616,7 +651,7 @@ config_global_load(void)
     /* ... window sizes */
     balsa_app.mw_width = gnome_config_get_int("MainWindowWidth=640");
     balsa_app.mw_height = gnome_config_get_int("MainWindowHeight=480");
-    balsa_app.mblist_width = gnome_config_get_int("MailboxListWidth=100");
+    balsa_app.mblist_width = gnome_config_get_int("MailboxListWidth=130");
     /* sendmsg window sizes */
     balsa_app.sw_width = gnome_config_get_int("SendMsgWindowWidth=640");
     balsa_app.sw_height = gnome_config_get_int("SendMsgWindowHeight=480");
@@ -654,15 +689,14 @@ config_global_load(void)
         g_free(tmp);
     }
 
-    /* ... Message window title format */
-    g_free(balsa_app.message_title_format);
-    balsa_app.message_title_format =
-        gnome_config_get_string("MessageTitleFormat="
-                                DEFAULT_MESSAGE_TITLE_FORMAT);
-
     balsa_app.expand_tree = gnome_config_get_bool("ExpandTree=false");
-    balsa_app.threading_type = d_get_gint("ThreadingType", 
-					  LB_MAILBOX_THREADING_JWZ);
+
+    {                             /* scope */
+        guint type =
+            gnome_config_get_int_with_default("ThreadingType", &def_used);
+        if (!def_used)
+            libbalsa_mailbox_set_threading_type(NULL, type);
+    }
 
     /* ... Quote colouring */
     g_free(balsa_app.quote_regex);
@@ -718,7 +752,7 @@ config_global_load(void)
 	gnome_config_get_bool("ConvertUnknown8Bit=false");
     balsa_app.convert_unknown_8bit_codeset =
 	gnome_config_get_int("ConvertUnknown8BitCodeset=" DEFAULT_BROKEN_CODESET);
-
+    libbalsa_set_fallback_codeset(balsa_app.convert_unknown_8bit_codeset);
     gnome_config_pop_prefix();
 
     /* Interface Options ... */
@@ -732,18 +766,29 @@ config_global_load(void)
     /* ... alternative layout of main window */
     balsa_app.alternative_layout = gnome_config_get_bool("AlternativeLayout=false");
     balsa_app.view_message_on_open = gnome_config_get_bool("ViewMessageOnOpen=true");
-    balsa_app.line_length = gnome_config_get_bool("MsgSizeAsLines=true");
     balsa_app.pgdownmod = gnome_config_get_bool("PageDownMod=false");
     balsa_app.pgdown_percent = gnome_config_get_int("PageDownPercent=50");
     if (balsa_app.pgdown_percent < 10)
 	balsa_app.pgdown_percent = 10;
+#if defined(ENABLE_TOUCH_UI)
+    balsa_app.do_file_format_check =
+        gnome_config_get_bool("FileFormatCheck=true");
+    balsa_app.enable_view_filter = gnome_config_get_bool("ViewFilter=false");
+#endif /* ENABLE_TOUCH_UI */
 
     /* ... Progress Window Dialog */
     balsa_app.pwindow_option = d_get_gint("ProgressWindow", WHILERETR);
-    balsa_app.delete_immediately =
-        gnome_config_get_bool("DeleteImmediately=false");
+
+    /* ... deleting messages: defaults enshrined here */
     balsa_app.hide_deleted =
         gnome_config_get_bool("HideDeleted=true");
+    libbalsa_mailbox_set_filter(NULL, balsa_app.hide_deleted ? 1 : 0);
+    balsa_app.expunge_on_close =
+        gnome_config_get_bool("ExpungeOnClose=true");
+    balsa_app.expunge_auto = gnome_config_get_bool("AutoExpunge=true");
+    /* timeout in hours */
+    balsa_app.expunge_timeout =
+        gnome_config_get_int("AutoExpungeHours=2") * 3600;
 
     gnome_config_pop_prefix();
 
@@ -811,9 +856,6 @@ config_global_load(void)
     /* Mailbox checking ... */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "MailboxList/");
 
-    /* ... color */
-    load_color("UnreadColor=" MBLIST_UNREAD_COLOR,
-	       &balsa_app.mblist_unread_color);
     /* ... show mailbox content info */
     balsa_app.mblist_show_mb_content_info =
 	gnome_config_get_bool("ShowMailboxContentInfo=true");
@@ -838,11 +880,13 @@ config_global_load(void)
     balsa_app.check_imap=d_get_gint("CheckIMAP", 1);
     balsa_app.check_imap_inbox=d_get_gint("CheckIMAPInbox", 0);
     balsa_app.quiet_background_check=d_get_gint("QuietBackgroundCheck", 0);
+    balsa_app.msg_size_limit=d_get_gint("POPMsgSizeLimit", 20000);
     gnome_config_pop_prefix();
 
-    /* IMAP folder scanning */
-    gnome_config_push_prefix(BALSA_CONFIG_PREFIX "IMAPFolderScanning/");
-    balsa_app.imap_scan_depth = d_get_gint("ScanDepth", 1);
+    /* folder scanning */
+    gnome_config_push_prefix(BALSA_CONFIG_PREFIX "FolderScanning/");
+    balsa_app.local_scan_depth = d_get_gint("LocalScanDepth", 1);
+    balsa_app.imap_scan_depth = d_get_gint("ImapScanDepth", 1);
     gnome_config_pop_prefix();
 
     /* how to react if a message with MDN request is displayed */
@@ -889,7 +933,8 @@ config_global_load(void)
         balsa_app.smtp_passphrase = 
             gnome_config_get_string("ESMTPPassphrase");
 #endif
-    balsa_app.smtp_tls_mode = gnome_config_get_int("ESMTPTLSMode=0");
+    /* default set to "Use TLS if possible" */
+    balsa_app.smtp_tls_mode = gnome_config_get_int("ESMTPTLSMode=1");
 #if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
     balsa_app.smtp_certificate_passphrase = 
         gnome_config_private_get_string("ESMTPCertificatePassphrase");
@@ -902,6 +947,9 @@ config_global_load(void)
 #endif
     /* ... outgoing mail */
     balsa_app.encoding_style = gnome_config_get_int("EncodingStyle=2");
+    /* convert libmutt Quoted Printable to GMime type, for compatible */
+    if (balsa_app.encoding_style==3)
+        balsa_app.encoding_style = GMIME_PART_ENCODING_QUOTEDPRINTABLE;
     balsa_app.wordwrap = gnome_config_get_bool("WordWrap=true");
     balsa_app.wraplength = gnome_config_get_int("WrapLength=72");
     if (balsa_app.wraplength < 40)
@@ -961,23 +1009,22 @@ config_global_load(void)
 	gnome_config_pop_prefix();
 	return FALSE;
     }
-    root_node =
+    balsa_app.root_node =
         balsa_mailbox_node_new_from_dir(balsa_app.local_mail_directory);
-    root_node->expanded = TRUE;
-    balsa_mailbox_nodes_lock(TRUE);
-    balsa_app.mailbox_nodes = g_node_new(root_node);
-    balsa_mailbox_nodes_unlock(TRUE);
 
-    balsa_app.open_inbox_upon_startup =
+#if defined(ENABLE_TOUCH_UI)
+     balsa_app.open_inbox_upon_startup =
+	gnome_config_get_bool("OpenInboxOnStartup=true");
+#else
+     balsa_app.open_inbox_upon_startup =
 	gnome_config_get_bool("OpenInboxOnStartup=false");
+#endif /* ENABLE_TOUCH_UI */
     /* debugging enabled */
     balsa_app.debug = gnome_config_get_bool("Debug=false");
 
     balsa_app.close_mailbox_auto = gnome_config_get_bool("AutoCloseMailbox=true");
     /* timeouts in minutes in config file for backwards compat */
     balsa_app.close_mailbox_timeout = gnome_config_get_int("AutoCloseMailboxTimeout=10") * 60;
-    balsa_app.commit_mailbox_auto = gnome_config_get_bool("AutoCommitMailbox=true");
-    balsa_app.commit_mailbox_timeout = gnome_config_get_int("AutoCommitMailboxTimeout=2") * 60;
     
     balsa_app.remember_open_mboxes =
 	gnome_config_get_bool("RememberOpenMailboxes=false");
@@ -1072,10 +1119,9 @@ config_save(void)
     gnome_config_set_string("DateFormat", balsa_app.date_string);
     gnome_config_set_int("ShownHeaders", balsa_app.shown_headers);
     gnome_config_set_string("SelectedHeaders", balsa_app.selected_headers);
-    gnome_config_set_string("MessageTitleFormat",
-                            balsa_app.message_title_format);
     gnome_config_set_bool("ExpandTree", balsa_app.expand_tree);
-    gnome_config_set_int("ThreadingType", balsa_app.threading_type);
+    gnome_config_set_int("ThreadingType",
+			 libbalsa_mailbox_get_threading_type(NULL));
     gnome_config_set_string("QuoteRegex", balsa_app.quote_regex);
     gnome_config_set_string("MessageFont", balsa_app.message_font);
     gnome_config_set_string("SubjectFont", balsa_app.subject_font);
@@ -1111,13 +1157,17 @@ config_save(void)
     gnome_config_set_int("ProgressWindow", balsa_app.pwindow_option);
     gnome_config_set_bool("AlternativeLayout", balsa_app.alternative_layout);
     gnome_config_set_bool("ViewMessageOnOpen", balsa_app.view_message_on_open);
-    gnome_config_set_bool("MsgSizeAsLines", balsa_app.line_length);
     gnome_config_set_bool("PageDownMod", balsa_app.pgdownmod);
     gnome_config_set_int("PageDownPercent", balsa_app.pgdown_percent);
-    gnome_config_set_bool("DeleteImmediately",
-                          balsa_app.delete_immediately);
-    gnome_config_set_bool("HideDeleted",
-                          balsa_app.hide_deleted);
+#if defined(ENABLE_TOUCH_UI)
+    gnome_config_set_bool("FileFormatCheck", balsa_app.do_file_format_check);
+    gnome_config_set_bool("ViewFilter",      balsa_app.enable_view_filter);
+#endif /* ENABLE_TOUCH_UI */
+    gnome_config_set_bool("HideDeleted", balsa_app.hide_deleted);
+    gnome_config_set_bool("ExpungeOnClose", balsa_app.expunge_on_close);
+    gnome_config_set_bool("AutoExpunge", balsa_app.expunge_auto);
+    gnome_config_set_int("AutoExpungeHours",
+                         balsa_app.expunge_timeout / 3600);
 
     gnome_config_pop_prefix();
 
@@ -1171,7 +1221,6 @@ config_save(void)
     /* Mailbox list options */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "MailboxList/");
 
-    save_color("UnreadColor", &balsa_app.mblist_unread_color);
     gnome_config_set_bool("ShowMailboxContentInfo",
 			  balsa_app.mblist_show_mb_content_info);
 
@@ -1191,12 +1240,14 @@ config_save(void)
     gnome_config_set_int("CheckIMAPInbox", balsa_app.check_imap_inbox);
     gnome_config_set_int("QuietBackgroundCheck",
 			 balsa_app.quiet_background_check);
+    gnome_config_set_int("POPMsgSizeLimit", balsa_app.msg_size_limit);
 
     gnome_config_pop_prefix();
 
-    /* IMAP folder scanning */
-    gnome_config_push_prefix(BALSA_CONFIG_PREFIX "IMAPFolderScanning/");
-    gnome_config_set_int("ScanDepth", balsa_app.imap_scan_depth);
+    /* folder scanning */
+    gnome_config_push_prefix(BALSA_CONFIG_PREFIX "FolderScanning/");
+    gnome_config_set_int("LocalScanDepth", balsa_app.local_scan_depth);
+    gnome_config_set_int("ImapScanDepth", balsa_app.imap_scan_depth);
     gnome_config_pop_prefix();
 
     /* how to react if a message with MDN request is displayed */
@@ -1258,8 +1309,6 @@ config_save(void)
 
     gnome_config_set_bool("AutoCloseMailbox", balsa_app.close_mailbox_auto);
     gnome_config_set_int("AutoCloseMailboxTimeout", balsa_app.close_mailbox_timeout/60);
-    gnome_config_set_bool("AutoCommitMailbox", balsa_app.commit_mailbox_auto);
-    gnome_config_set_int("AutoCommitMailboxTimeout", balsa_app.commit_mailbox_timeout/60);
 
     gnome_config_set_bool("RememberOpenMailboxes",
 			  balsa_app.remember_open_mboxes);
@@ -1367,6 +1416,13 @@ config_address_books_load(void)
     g_free(tmp);
     gnome_config_pop_prefix();
 
+    /* Free old data in case address books were set by eg. config druid. */
+    if(balsa_app.address_book_list) {
+        g_list_foreach(balsa_app.address_book_list,
+                       (GFunc)g_object_unref, NULL);
+        g_list_free(balsa_app.address_book_list);
+        balsa_app.address_book_list = NULL;
+    }
     iterator = gnome_config_init_iterator_sections(BALSA_CONFIG_PREFIX);
     while ((iterator = gnome_config_iterator_next(iterator, &key, &val))) {
 
@@ -1413,6 +1469,12 @@ config_identities_load()
     gchar *key, *val, *tmp;
     int pref_len = strlen(IDENTITY_SECTION_PREFIX);
 
+    /* Free old data in case identities were set by eg. config druid. */
+    if(balsa_app.identities) {
+        g_list_foreach(balsa_app.identities, (GFunc)g_object_unref, NULL);
+        g_list_free(balsa_app.identities);
+        balsa_app.identities = NULL;
+    }
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "identity/");
     default_ident = gnome_config_get_string("CurrentIdentity");
     gnome_config_pop_prefix();
@@ -1443,6 +1505,8 @@ config_identities_load()
 	    LIBBALSA_IDENTITY(balsa_app.identities->data);
 
     g_free(default_ident);
+    if(balsa_app.main_window)
+        balsa_identities_changed(balsa_app.main_window);
 }
 
 /* config_identities_save:
@@ -1477,28 +1541,34 @@ config_identities_save(void)
     }
 }
 
-void
-config_views_load(void)
+static void
+config_views_load_with_prefix(const gchar * prefix, gboolean compat)
 {
     void *iterator;
     gchar *key, *val, *tmp;
-    int pref_len = strlen(VIEW_SECTION_PREFIX);
+    int pref_len = strlen(prefix);
     int def;
 
     iterator = gnome_config_init_iterator_sections(BALSA_CONFIG_PREFIX);
     while ((iterator = gnome_config_iterator_next(iterator, &key, &val))) {
-	if (strncmp(key, VIEW_SECTION_PREFIX, pref_len) == 0) {
+	if (strncmp(key, prefix, pref_len) == 0) {
 	    gchar *url;
 	    tmp = g_strconcat(BALSA_CONFIG_PREFIX, key, "/", NULL);
 	    gnome_config_push_prefix(tmp);
 	    g_free(tmp);
-	    url = gnome_config_get_string_with_default("URL", &def);
-	    if (!def) {
+	    url = compat
+		? gnome_config_get_string_with_default("URL", &def)
+		: libbalsa_urldecode(&key[pref_len]);
+	    if (!compat || !def) {
                 LibBalsaMailboxView *view;
+		gint tmp;
                 gchar *address;
 
                 view = libbalsa_mailbox_view_new();
-                g_hash_table_insert(balsa_app.mailbox_views,
+		/* In compatibility mode, mark as not in sync, to force
+		 * the view to be saved in the config file. */
+		view->in_sync = !compat;
+                g_hash_table_insert(libbalsa_mailbox_view_table,
                                     g_strdup(url), view);
 
                 address =
@@ -1510,35 +1580,33 @@ config_views_load(void)
 
                 view->identity_name = gnome_config_get_string("Identity");
 
-                view->threading_type =
-                    gnome_config_get_int_with_default("Threading", &def);
-                if (def)
-                    view->threading_type = LB_MAILBOX_THREADING_SIMPLE;
+		tmp = gnome_config_get_int_with_default("Threading", &def);
+                if (!def)
+		    view->threading_type = tmp;
 
-                view->sort_type =
-                    gnome_config_get_int_with_default("SortType", &def);
-                if (def)
-                    view->sort_type = LB_MAILBOX_SORT_TYPE_ASC;
+		tmp = gnome_config_get_int_with_default("GUIFilter", &def);
+                if (!def)
+		    view->filter = tmp;
 
-                view->sort_field =
-                    gnome_config_get_int_with_default("SortField", &def);
-                if (def)
-                    view->sort_field = LB_MAILBOX_SORT_DATE;
+		tmp = gnome_config_get_int_with_default("SortType", &def);
+                if (!def)
+		    view->sort_type = tmp;
 
-                view->show =
-                    gnome_config_get_int_with_default("Show", &def);
-                if (def)
-                    view->show = LB_MAILBOX_SHOW_UNSET;
+		tmp = gnome_config_get_int_with_default("SortField", &def);
+                if (!def)
+		    view->sort_field = tmp;
 
-                view->exposed =
-                    gnome_config_get_bool_with_default("Exposed", &def);
-                if (def)
-                    view->exposed = FALSE;
+		tmp = gnome_config_get_int_with_default("Show", &def);
+                if (!def)
+		    view->show = tmp;
 
-                view->open =
-                    gnome_config_get_bool_with_default("Open", &def);
-                if (def)
-                    view->open = FALSE;
+		tmp = gnome_config_get_bool_with_default("Exposed", &def);
+                if (!def)
+		    view->exposed = tmp;
+
+		tmp = gnome_config_get_bool_with_default("Open", &def);
+                if (!def)
+		    view->open = tmp;
             }
             gnome_config_pop_prefix();
             g_free(url);
@@ -1548,58 +1616,71 @@ config_views_load(void)
     }
 }
 
-/* config_views_save:
-   iterates over all mailboxes and save the views.
-*/
-static gboolean
-save_view(GNode * node, int *cnt)
+void
+config_views_load(void)
 {
-    gchar *prefix;
-    LibBalsaMailboxView *view;
-    BalsaMailboxNode* mn = BALSA_MAILBOX_NODE(node->data);
-    g_return_val_if_fail(mn, FALSE);
-    
-    if(!mn->mailbox) return FALSE;
+    /* Load old-style config sections in compatibility mode. */
+    config_views_load_with_prefix(VIEW_SECTION_PREFIX, TRUE);
+    config_views_load_with_prefix(VIEW_BY_URL_SECTION_PREFIX, FALSE);
+}
 
-    view = mn->mailbox->view;
-    prefix = g_strdup_printf("%s%d/",
-			     BALSA_CONFIG_PREFIX VIEW_SECTION_PREFIX, 
-			     ++(*cnt));
+/* config_views_save:
+   iterates over all mailbox views.
+*/
+static void
+save_view(const gchar * url, LibBalsaMailboxView * view)
+{
+    gchar *url_enc;
+    gchar *prefix;
+
+    if (!view || view->in_sync)
+	return;
+    view->in_sync = TRUE;
+
+    url_enc = libbalsa_urlencode(url);
+    prefix = g_strconcat(BALSA_CONFIG_PREFIX VIEW_BY_URL_SECTION_PREFIX,
+			 url_enc, "/", NULL);
+    g_free(url_enc);
+
+    gnome_config_clean_section(prefix);
     gnome_config_push_prefix(prefix);
     g_free(prefix);
-    gnome_config_set_string("URL", mn->mailbox->url);
-    if (view->mailing_list_address) {
+
+    if (view->mailing_list_address !=
+	libbalsa_mailbox_get_mailing_list_address(NULL)) {
        gchar* tmp = libbalsa_address_to_gchar(view->mailing_list_address, 0);
        gnome_config_set_string("MailingListAddress", tmp);
        g_free(tmp);
-    } else {
-       gnome_config_clean_key("MailingListAddress");
     }
-    if(view->identity_name)
-       gnome_config_set_string("Identity", view->identity_name);
-    else gnome_config_clean_key("Identity");
-    gnome_config_set_int("Threading",   view->threading_type);
-    gnome_config_set_int("SortType",    view->sort_type);
-    gnome_config_set_int("SortField",   view->sort_field);
-    gnome_config_set_int("Show", view->show);
-    gnome_config_set_bool("Exposed", view->exposed);
-    gnome_config_set_bool("Open", view->open);
+    if (view->identity_name  != libbalsa_mailbox_get_identity_name(NULL))
+	gnome_config_set_string("Identity", view->identity_name);
+    if (view->threading_type != libbalsa_mailbox_get_threading_type(NULL))
+	gnome_config_set_int("Threading",   view->threading_type);
+    if (view->filter         != libbalsa_mailbox_get_filter(NULL))
+	gnome_config_set_int("GUIFilter",   view->filter);
+    if (view->sort_type      != libbalsa_mailbox_get_sort_type(NULL))
+	gnome_config_set_int("SortType",    view->sort_type);
+    if (view->sort_field     != libbalsa_mailbox_get_sort_field(NULL))
+	gnome_config_set_int("SortField",   view->sort_field);
+    /* initial value for show is UNSET, but that's always replaced, 
+     * and we want it to default to FROM. */
+    if (view->show           != LB_MAILBOX_SHOW_FROM)
+	gnome_config_set_int("Show",	    view->show);
+    if (view->exposed        != libbalsa_mailbox_get_exposed(NULL))
+	gnome_config_set_bool("Exposed",    view->exposed);
+    if (view->open           != libbalsa_mailbox_get_open(NULL))
+	gnome_config_set_bool("Open",	    view->open);
 
     gnome_config_pop_prefix();
-    return FALSE;
 }
 
 void
 config_views_save(void)
 {
-    int cnt = 0;
-
     config_clean_sections(VIEW_SECTION_PREFIX);
     /* save current */
-    balsa_mailbox_nodes_lock(FALSE);
-    g_node_traverse(balsa_app.mailbox_nodes, G_IN_ORDER, G_TRAVERSE_ALL, -1,
-		   (GNodeTraverseFunc) save_view, &cnt);
-    balsa_mailbox_nodes_unlock(FALSE);
+    g_hash_table_foreach(libbalsa_mailbox_view_table, (GHFunc) save_view,
+			 NULL);
 }
 
 static void
@@ -1620,7 +1701,7 @@ config_filters_load(void)
     void *iterator;
     gchar * key,* tmp;
     gint pref_len = strlen(FILTER_SECTION_PREFIX);
-    gint non_critical_error=FILTER_NOERR;
+    guint save = 0;
 
     filter_errno=FILTER_NOERR;
     iterator = gnome_config_init_iterator_sections(BALSA_CONFIG_PREFIX);
@@ -1632,32 +1713,33 @@ config_filters_load(void)
 	    gnome_config_push_prefix(tmp);
 	    g_free(tmp);	    
 	    fil = libbalsa_filter_new_from_config();
-	    FILTER_SETFLAG(fil,FILTER_VALID);
-	    FILTER_SETFLAG(fil,FILTER_COMPILED);
-	    gnome_config_pop_prefix();
-
-	    if (fil) {
-		libbalsa_conditions_new_from_config(BALSA_CONFIG_PREFIX,key,fil);
-		if (filter_errno==FILTER_EFILESYN) {
-		    /* Don't abort on syntax error, just remember it
-		     * FIXME : we could make a GSList of errors to report them to user 
-		     */
-		    non_critical_error=FILTER_EFILESYN;
-		    filter_errno=FILTER_NOERR;
-		}
-		else if (filter_errno!=FILTER_NOERR) {
-		    /* Critical error, we abort the loading process
-		     */
-		    libbalsa_filter_free(fil,NULL);
-		    return;
-		}
-		balsa_app.filters = g_slist_prepend(balsa_app.filters,(gpointer)fil);
+	    if (!fil->condition) {
+		/* Try pre-2.1 style: */
+		FilterOpType op = gnome_config_get_int("Operation");
+		ConditionMatchType cmt =
+		    op == FILTER_OP_OR ? CONDITION_OR : CONDITION_AND;
+		gnome_config_pop_prefix();
+		fil->condition =
+		    libbalsa_condition_new_2_0(BALSA_CONFIG_PREFIX, key, cmt);
+		gnome_config_push_prefix(tmp);
+		if (fil->condition)
+		    ++save;
 	    }
+	    if (!fil->condition) {
+		g_idle_add((GSourceFunc) config_warning_idle,
+			   _("Filter with no condition was ignored"));
+		libbalsa_filter_free(fil, GINT_TO_POINTER(FALSE));
+	    } else {
+		FILTER_SETFLAG(fil,FILTER_VALID);
+		FILTER_SETFLAG(fil,FILTER_COMPILED);
+		balsa_app.filters = g_slist_prepend(balsa_app.filters, fil);
+	    }
+	    gnome_config_pop_prefix();
 	}
 	g_free(key);	
     }
-    if (filter_errno==FILTER_NOERR)
-	filter_errno=non_critical_error;
+    if (save)
+	config_filters_save();
 }
 
 #define FILTER_SECTION_MAX "9999"
@@ -1683,13 +1765,6 @@ config_filters_save(void)
 	gnome_config_push_prefix(buffer);
 	libbalsa_filter_save_config(fil);
 	gnome_config_pop_prefix();
-
-	/* We suppress the final "/", this is necessary in order that
-	 * libbalsa_conditions_save_config can construct the condition
-	 * section name */
-	tmp[i-1]='\0';
-	libbalsa_conditions_save_config(fil->conditions,
-					BALSA_CONFIG_PREFIX,section_name);
     }
     gnome_config_sync();
     /* This loop takes care of cleaning up old filter sections */
@@ -1697,12 +1772,6 @@ config_filters_save(void)
 	i=snprintf(tmp,tmp_len,"%d/",nb++);
 	if (gnome_config_has_section(buffer)) {
 	    gnome_config_clean_section(buffer);
-	    /* We suppress the final "/", this is necessary in order
-	     * that libbalsa_clean_condition_sections can construct
-	     * the condition section name */
-	    tmp[i-1]='\0';
-	    libbalsa_clean_condition_sections(BALSA_CONFIG_PREFIX,
-					      section_name);
 	}
 	else break;
     }
@@ -1710,7 +1779,8 @@ config_filters_save(void)
     g_free(buffer);
 }
 
-void config_mailbox_filters_save(LibBalsaMailbox * mbox)
+void
+config_mailbox_filters_save(LibBalsaMailbox * mbox)
 {
     gchar * tmp;
 
