@@ -129,8 +129,8 @@ button_data toolbar_buttons[]={
      N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {GTK_STOCK_SPELL_CHECK, N_("Spelling"),
      N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GTK_STOCK_CLOSE, N_("Cancel"), 
-     N_("Cancel this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {GTK_STOCK_CLOSE, N_("Close"), 
+     N_("Close the compose window"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_MARKED_NEW, N_("Toggle\nnew"),
      N_("Toggle new message flag"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_MARKED_ALL, N_("Mark all"),
@@ -345,6 +345,9 @@ balsa_toolbar_refresh(GtkWidget * toolbar)
                                            GTK_ICON_SIZE_LARGE_TOOLBAR),
                                        GTK_SIGNAL_FUNC(bti->callback),
                                        bti->user_data);
+	g_object_add_weak_pointer(G_OBJECT(bti->widget),
+				  (gpointer) &bti->widget);
+        GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(bti->widget), GTK_CAN_FOCUS);
         g_free(text);
         gtk_widget_set_sensitive(bti->widget, bti->sensitive);
         if (type == GTK_TOOLBAR_CHILD_TOGGLEBUTTON) {
@@ -456,6 +459,16 @@ bt_destroy(GtkWidget * toolbar, BalsaToolbarData * btd)
 /* Create and initialize a BalsaToolbarData structure with the given
  * BalsaToolbarModel.
  */
+static void
+bt_free_icon(gpointer data)
+{
+    BalsaToolbarIcon *bti = data;
+    if (bti->widget)
+	g_object_remove_weak_pointer(G_OBJECT(bti->widget),
+				     (gpointer) &bti->widget);
+    g_free(bti);
+}
+
 static BalsaToolbarData *
 bt_data_new(BalsaToolbarModel * model)
 {
@@ -463,7 +476,7 @@ bt_data_new(BalsaToolbarModel * model)
 
     btd->model = model;
     btd->table =
-        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, bt_free_icon);
 
     return btd;
 }
